import prisma from "@/lib/prisma";
import { Tone } from "@/types/tone";
import { TonalPalette } from "@material/material-color-utilities";
import crypto from 'crypto';

export function generatePaletteName(tones: Tone[]): string {
  const hexString = tones.map(t => t.hex.toLowerCase()).join('');
  const hash = crypto.createHash('sha256').update(hexString).digest('hex');
  return `palette-${hash.slice(0, 8)}`;
}

export function rgbToHex(rgb: number[]): string {
  return `#${rgb
    .map((val) => Math.round(val).toString(16).padStart(2, "0"))
    .join("")}`;
}

export function argbToHex(argb: number): string {
  return `#${argb.toString(16).slice(2).padStart(6, "0")}`;
}
  
export function extractPaletteTones(
  palette: TonalPalette, tones: number[] = [0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100]
) {
  return tones.map((t) => ({
    tone: t,
    hex: argbToHex(palette.tone(t)),
  }));
}

export async function upsertPalettes(tones: Tone[], imageId: string, type: string) {
  const paletteName = generatePaletteName(tones);

  const existingPalette = await prisma.colorPalette.findFirst({
    where: { name: paletteName },
    include: { images: { select: { id: true } } },
  });

  if (existingPalette) {
    const alreadyLinked = existingPalette.images.some(img => img.id === imageId);

    if (!alreadyLinked) {
      await prisma.colorPalette.update({
        where: { id: existingPalette.id },
        data: {
          images: {
            connect: { id: imageId },
          },
        },
      });
    }

    return existingPalette;
  }

  const newPalette = await prisma.colorPalette.create({
    data: {
      name: paletteName,
      type: type,
      items: {
        create: tones.map(t => ({
          tone: t.tone,
          hex: t.hex,
        })),
      },
      images: {
        connect: { id: imageId },
      },
    },
    include: { items: true },
  });

  return newPalette;
}